@@ -134,6 +134,23 @@ class ModelBand(object):
         self._model = model
         self._cov_param = cov_param
         self._param = param
+        self._error_map = error_map
+
+    @property
+    def model(self):
+        """
+
+        :return: model, 2d numpy array
+        """
+        return self._model
+
+    @property
+    def norm_residuals(self):
+        """
+
+        :return: normalized residuals, 2d numpy array
+        """
+        return self._norm_residuals
 
     @property
     def image_model_class(self):
@@ -166,7 +183,7 @@ def check_solver_error(image):
     :return: bool, True if solver could not find a unique solution, False if solver works
     """
     result = np.all(image == 0)
-    if result is True:
+    if result:
         Warning('Linear inversion of surface brightness components did not result in a unique solution.'
                 'All linear amplitude parameters are set =0 instead. Please check whether '
                 'a) there are too many basis functions in the model, '
