@@ -130,3 +130,14 @@ class Hernquist_Ellipse(LensProfileBase):
         :return: enclosed mass
         """
         return self.spherical.mass_3d(r, rho0, Rs)
+
+    def mass_3d_lens(self, r, sigma0, Rs, e1=0, e2=0):
+        """
+        mass enclosed a 3d sphere or radius r in lensing parameterization
+
+        :param r: 3-d radius within the mass is integrated (same distance units as density definition)
+        :param sigma0: rho0 * Rs (units of projected density)
+        :param Rs: Hernquist radius
+        :return: enclosed mass
+        """
+        return self.spherical.mass_3d_lens(r, sigma0, Rs)
