@@ -1,7 +1,5 @@
 from lenstronomy.LightModel.Profiles.nie import NIE
 from lenstronomy.LensModel.Profiles.chameleon import Chameleon as ChameleonLens
-import lenstronomy.Util.param_util as param_util
-
 from lenstronomy.Util.package_util import exporter
 export, __all__ = exporter()
 
@@ -9,7 +7,7 @@ export, __all__ = exporter()
 @export
 class Chameleon(object):
     """
-    class of the Chameleon model (See Suyu+2014) an elliptical truncated double isothermal profile
+    class of the Chameleon model (See Dutton+ 2011, Suyu+2014) an elliptical truncated double isothermal profile
 
     """
     param_names = ['amp', 'w_c', 'w_t', 'e1', 'e2', 'center_x', 'center_y']
@@ -25,28 +23,45 @@ class Chameleon(object):
 
         :param x: ra-coordinate
         :param y: dec-coordinate
+        :param w_c:
+        :param w_t:
         :param amp: amplitude of first power-law flux
-        :param flux_ratio: ratio of amplitudes of first to second power-law profile
-        :param gamma1: power-law slope
-        :param gamma2: power-law slope
-        :param e1: ellipticity parameter
-        :param e2: ellipticity parameter
+        :param e1: eccentricity parameter
+        :param e2: eccentricity parameter
         :param center_x: center
         :param center_y: center
         :return: flux of chameleon profile
         """
         amp_new, w_c, w_t, s_scale_1, s_scale_2 = self._chameleonLens.param_convert(amp, w_c, w_t, e1, e2)
-        phi_G, q = param_util.ellipticity2phi_q(e1, e2)
         flux1 = self.nie.function(x, y, 1, e1, e2, s_scale_1, center_x, center_y)
         flux2 = self.nie.function(x, y, 1, e1, e2, s_scale_2, center_x, center_y)
-        flux = amp_new / (1. + q) * (flux1 - flux2)
+        flux = amp_new * (flux1 - flux2)
+        return flux
+
+    def light_3d(self, r, amp, w_c, w_t, e1, e2, center_x=0, center_y=0):
+        """
+
+        :param r: 3d radius
+        :param w_c:
+        :param w_t:
+        :param amp: amplitude of first power-law flux
+        :param e1: eccentricity parameter
+        :param e2: eccentricity parameter
+        :param center_x: center
+        :param center_y: center
+        :return: 3d flux of chameleon profile at radius r
+        """
+        amp_new, w_c, w_t, s_scale_1, s_scale_2 = self._chameleonLens.param_convert(amp, w_c, w_t, e1, e2)
+        flux1 = self.nie.light_3d(r, 1, e1, e2, s_scale_1, center_x, center_y)
+        flux2 = self.nie.light_3d(r, 1, e1, e2, s_scale_2, center_x, center_y)
+        flux = amp_new * (flux1 - flux2)
         return flux
 
 
 @export
 class DoubleChameleon(object):
     """
-    class of the Chameleon model (See Suyu+2014) an elliptical truncated double isothermal profile
+    class of the double Chameleon model. See Dutton+2011, Suyu+2014 for the single Chameleon model.
 
     """
     param_names = ['amp', 'ratio', 'w_c1', 'w_t1', 'e11', 'e21', 'w_c2', 'w_t2', 'e12', 'e22', 'center_x', 'center_y']
@@ -81,6 +96,28 @@ class DoubleChameleon(object):
         f_2 = self.chameleon.function(x, y, amp / (1. + ratio), w_c2, w_t2, e12, e22, center_x, center_y)
         return f_1 + f_2
 
+    def light_3d(self, r, amp, ratio, w_c1, w_t1, e11, e21, w_c2, w_t2, e12, e22, center_x=0, center_y=0):
+        """
+
+        :param r: 3d radius
+        :param amp:
+        :param ratio: ratio of first to second amplitude of Chameleon surface brightness
+        :param w_c1:
+        :param w_t1:
+        :param e11:
+        :param e21:
+        :param w_c2:
+        :param w_t2:
+        :param e12:
+        :param e22:
+        :param center_x:
+        :param center_y:
+        :return: 3d light density at radius r
+        """
+        f_1 = self.chameleon.light_3d(r, amp / (1. + 1./ratio), w_c1, w_t1, e11, e21, center_x, center_y)
+        f_2 = self.chameleon.light_3d(r, amp / (1. + ratio), w_c2, w_t2, e12, e22, center_x, center_y)
+        return f_1 + f_2
+
 
 @export
 class TripleChameleon(object):
@@ -111,7 +148,7 @@ class TripleChameleon(object):
 
         :param amp:
         :param ratio12: ratio of first to second amplitude
-        :param ratio13: ratio of first to third amplidute
+        :param ratio13: ratio of first to third amplitude
         :param w_c1:
         :param w_t1:
         :param e11:
@@ -120,6 +157,10 @@ class TripleChameleon(object):
         :param w_t2:
         :param e12:
         :param e22:
+        :param w_c3:
+        :param w_t3:
+        :param e13:
+        :param e23:
         :param center_x:
         :param center_y:
         :return:
@@ -129,5 +170,37 @@ class TripleChameleon(object):
         amp3 = amp1 / ratio13
         f_1 = self.chameleon.function(x, y, amp1, w_c1, w_t1, e11, e21, center_x, center_y)
         f_2 = self.chameleon.function(x, y, amp2, w_c2, w_t2, e12, e22, center_x, center_y)
-        f_3 = self.chameleon.function(x, y, amp3, w_c2, w_t2, e12, e22, center_x, center_y)
+        f_3 = self.chameleon.function(x, y, amp3, w_c3, w_t3, e13, e23, center_x, center_y)
+        return f_1 + f_2 + f_3
+
+    def light_3d(self, r, amp, ratio12, ratio13, w_c1, w_t1, e11, e21, w_c2, w_t2, e12, e22, w_c3, w_t3, e13, e23,
+                 center_x=0, center_y=0):
+        """
+
+        :param r: 3d light radius
+        :param amp:
+        :param ratio12: ratio of first to second amplitude
+        :param ratio13: ratio of first to third amplitude
+        :param w_c1:
+        :param w_t1:
+        :param e11:
+        :param e21:
+        :param w_c2:
+        :param w_t2:
+        :param e12:
+        :param e22:
+        :param w_c3:
+        :param w_t3:
+        :param e13:
+        :param e23:
+        :param center_x:
+        :param center_y:
+        :return:
+        """
+        amp1 = amp / (1. + 1./ratio12 + 1./ratio13)
+        amp2 = amp1 / ratio12
+        amp3 = amp1 / ratio13
+        f_1 = self.chameleon.light_3d(r, amp1, w_c1, w_t1, e11, e21, center_x, center_y)
+        f_2 = self.chameleon.light_3d(r, amp2, w_c2, w_t2, e12, e22, center_x, center_y)
+        f_3 = self.chameleon.light_3d(r, amp3, w_c3, w_t3, e13, e23, center_x, center_y)
         return f_1 + f_2 + f_3
