@@ -13,17 +13,53 @@ export, __all__ = exporter()
 @export
 class Shapelets(object):
     """
+    class for 2d cartesian Shapelets.
+
+    Sources:
+    Refregier 2003: Shapelets: I. A Method for Image Analysis https://arxiv.org/abs/astro-ph/0105178
+    Refregier 2003: Shapelets: II. A Method for Weak Lensing Measurements https://arxiv.org/abs/astro-ph/0105179
+
+    For one dimension, the shapelets are defined as
+
+    .. math::
+        \\phi_n(x) \\equiv \\left[2^n \\pi^{1/2} n!  \\right]]^{-1/2}H_n(x) e^{-\\frac{x^2}{2}}
+
+    This basis is orthonormal. The dimensional basis function is
+
+    .. math::
+        B_n(x;\\beta) \equiv \\beta^{-1/2} \\phi_n(\\beta^{-1}x)
+
+    which are orthonormal as well.
+
+    The two-dimensional basis function is
+
+    .. math::
+        \\phi_{\\bf n}({\bf x}) \equiv \\phi_{n1}(x1) \\phi_{n2}(x2)
+
+    where :math:`{\\bf n} \\equiv (n1, n2)` and :math:`{\\bf x} \\equiv (x1, x2)`.
+
+    The dimensional two-dimentional basis function is
+
+    .. math::
+        B_{\\bf n}({\\bf x};\\beta) \equiv \\beta^{-1/2} \\phi_{\\bf n}(\\beta^{-1}{\\bf x}).
 
     """
     param_names = ['amp', 'beta', 'n1', 'n2', 'center_x', 'center_y']
-    lower_limit_default = {'amp': 0, 'beta': 0, 'n1': 0, 'n2': 0, 'center_x': -100, 'center_y': -100}
+    lower_limit_default = {'amp': 0, 'beta': 0.01, 'n1': 0, 'n2': 0, 'center_x': -100, 'center_y': -100}
     upper_limit_default = {'amp': 100, 'beta': 100, 'n1': 150, 'n2': 150, 'center_x': 100, 'center_y': 100}
 
     def __init__(self, interpolation=False, precalc=False, stable_cut=True, cut_scale=5):
         """
         load interpolation of the Hermite polynomials in a range [-30,30] in order n<= 150
-        :return:
+
+        :param interpolation: boolean; if True, uses interpolated pre-calculated shapelets in the evaluation
+        :param precalc: boolean; if True interprets as input (x, y) as pre-calculated normalized shapelets
+        :param stable_cut: boolean; if True, sets the values outside of
+         :math:`\\sqrt\\left(n_{\\rm max} + 1 \\right) \\beta s_{\\rm cut scale} = 0`.
+        :param cut_scale: float, scaling parameter where to cut the shapelets. This is for numerical reasons such that
+         the polynomials in the Hermite function do not get unstable.
         """
+
         self._interpolation = interpolation
         self._precalc = precalc
         self._stable_cut = stable_cut
@@ -46,7 +82,7 @@ class Shapelets(object):
 
         :param x: array of values
         :param n_array: list of coeffs in H_n
-        :param cut_scale: scale where the polynomial will be set to zero
+        :param tensor: see numpy.polynomial.hermite.hermval
         :return: see numpy.polynomial.hermite.hermval
         """
         if not self._stable_cut:
@@ -66,14 +102,17 @@ class Shapelets(object):
 
     def function(self, x, y, amp, beta, n1, n2, center_x, center_y):
         """
+        2d cartesian shapelet
 
+        :param x: x-coordinate
+        :param y: y-coordinate
         :param amp: amplitude of shapelet
         :param beta: scale factor of shapelet
-        :param n1: x-order
-        :param n2: y-order
+        :param n1: x-order of Hermite polynomial
+        :param n2: y-order of Hermite polynomial
         :param center_x: center in x
         :param center_y: center in y
-        :return:
+        :return: flux surface brighness at (x, y)
         """
 
         if self._precalc:
@@ -89,9 +128,8 @@ class Shapelets(object):
         :param n: The n'the basis function.
         :type name: int.
         :param x: 1-dim position (dimensionless)
-        :type state: float or numpy array.
+        :type x: float or numpy array.
         :returns:  array-- H_n(x).
-        :raises: AttributeError, KeyError
         """
         if not self._interpolation:
             n_array = np.zeros(n+1)
@@ -107,23 +145,22 @@ class Shapelets(object):
         :param n: The n'the basis function.
         :type name: int.
         :param x: 1-dim position (dimensionless)
-        :type state: float or numpy array.
+        :type x: float or numpy array.
         :returns:  array-- phi_n(x).
-        :raises: AttributeError, KeyError
         """
         prefactor = 1./np.sqrt(2**n*np.sqrt(np.pi)*math.factorial(n))
         return prefactor*self.H_n(n, x)*np.exp(-x**2/2.)
 
     def pre_calc(self, x, y, beta, n_order, center_x, center_y):
         """
-        calculates the H_n(x) and H_n(y) for a given x-array and y-array
-        :param x:
-        :param y:
-        :param amp:
-        :param beta:
-        :param n_order:
-        :param center_x:
-        :param center_y:
+        calculates the H_n(x) and H_n(y) for a given x-array and y-array for the full order in the polynomials
+
+        :param x: x-coordinates (numpy array)
+        :param y: 7-coordinates (numpy array)
+        :param beta: shapelet scale
+        :param n_order: order of shapelets
+        :param center_x: shapelet center
+        :param center_y: shapelet center
         :return: list of H_n(x) and H_n(y)
         """
         x_ = x - center_x
@@ -131,25 +168,26 @@ class Shapelets(object):
         n = len(np.atleast_1d(x))
         H_x = np.empty((n_order+1, n))
         H_y = np.empty((n_order+1, n))
+        exp_x = np.exp(-(x_ / beta) ** 2 / 2.)
+        exp_y = np.exp(-(y_ / beta) ** 2 / 2.)
         if n_order > 170:
             raise ValueError('polynomial order to large', n_order)
         for n in range(0, n_order+1):
-
             prefactor = 1./np.sqrt(2**n*np.sqrt(np.pi)*math.factorial(n))
             n_array = np.zeros(n+1)
             n_array[n] = 1
-            H_x[n] = self.hermval(x_/beta, n_array, tensor=False) * prefactor * np.exp(-(x_/beta)**2/2.)
-            H_y[n] = self.hermval(y_/beta, n_array, tensor=False) * prefactor * np.exp(-(y_/beta)**2/2.)
+            H_x[n] = self.hermval(x_/beta, n_array, tensor=False) * prefactor * exp_x
+            H_y[n] = self.hermval(y_/beta, n_array, tensor=False) * prefactor * exp_y
         return H_x, H_y
 
 
 @export
 class ShapeletSet(object):
     """
-    class to operate on entire shapelet set
+    class to operate on entire shapelet set limited by a maximal polynomial order n_max, such that n1 + n2 <= n_max
     """
     param_names = ['amp', 'n_max', 'beta', 'center_x', 'center_y']
-    lower_limit_default = {'beta': 0, 'center_x': -100, 'center_y': -100}
+    lower_limit_default = {'beta': 0.01, 'center_x': -100, 'center_y': -100}
     upper_limit_default = {'beta': 100, 'center_x': 100, 'center_y': 100}
 
     def __init__(self):
@@ -158,13 +196,14 @@ class ShapeletSet(object):
     def function(self, x, y, amp, n_max, beta, center_x=0, center_y=0):
         """
 
-        :param x:
-        :param y:
-        :param amp:
-        :param beta:
-        :param center_x:
-        :param center_y:
-        :return:
+        :param x: x-coordinates
+        :param y: y-coordinates
+        :param amp: array of amplitudes in pre-defined order of shapelet basis functions
+        :param beta: shapelet scale
+        :param n_max: maximum polynomial order in Hermite polynomial
+        :param center_x: shapelet center
+        :param center_y: shapelet center
+        :return: surface brightness of combined shapelet set
         """
         num_param = int((n_max+1)*(n_max+2)/2)
         f_ = np.zeros(len(np.atleast_1d(x)))
@@ -181,13 +220,25 @@ class ShapeletSet(object):
             else:
                 n1 -= 1
                 n2 += 1
-        try: len(x)
-        except: f_ = f_[0]
-        #if isinstance(x, int) or isinstance(x, float):
-        #    f_ = f_[0]
+        try:
+            len(x)
+        except:
+            f_ = f_[0]
         return np.nan_to_num(f_)
 
     def function_split(self, x, y, amp, n_max, beta, center_x=0, center_y=0):
+        """
+        splits shapelet set in list of individual shapelet basis function responses
+
+        :param x: x-coordinates
+        :param y: y-coordinates
+        :param amp: array of amplitudes in pre-defined order of shapelet basis functions
+        :param beta: shapelet scale
+        :param n_max: maximum polynomial order in Hermite polynomial
+        :param center_x: shapelet center
+        :param center_y: shapelet center
+        :return: list of individual shapelet basis function responses
+        """
         num_param = int((n_max+1)*(n_max+2)/2)
         A = []
         n1 = 0
