@@ -120,6 +120,42 @@ class TestLensProfileAnalysis(object):
         npt.assert_almost_equal(center_x_out, center_x, 2)
         npt.assert_almost_equal(center_y_out, center_y, 2)
 
+    def test_mst_invariant_differential(self):
+
+        # testing with a SPP profile
+        lensModel = LensModel(**{'lens_model_list': ['SPP']})
+        profileAnalysis = LensProfileAnalysis(lens_model=lensModel)
+        gamma_list = [2, 1.8, 2.2]
+        theta_E_list = [1, 0.5, 2]
+        for gamma in gamma_list:
+            for theta_E in theta_E_list:
+                kwargs_lens = [{'theta_E': theta_E, 'gamma': gamma, 'center_x': 0, 'center_y': 0}]
+                xi = profileAnalysis.mst_invariant_differential(kwargs_lens, radius=theta_E, center_x=None,
+                                                                center_y=None, model_list_bool=None, num_points=10)
+
+                xi_true = (gamma - 2) / theta_E
+                npt.assert_almost_equal(xi, xi_true, decimal=3)
+
+        # it should also work in the elliptical regime
+        lensModel = LensModel(**{'lens_model_list': ['EPL']})
+        profileAnalysis = LensProfileAnalysis(lens_model=lensModel)
+        gamma_list = [2, 1.8, 2.2]
+        theta_E_list = [1, 0.5, 2]
+        q_list = [1, 0.9, 0.8, 0.7]
+        for gamma in gamma_list:
+            for theta_E in theta_E_list:
+                    for q in q_list:
+                        e1, e2 = param_util.phi_q2_ellipticity(phi=0, q=q)
+                        kwargs_lens = [{'theta_E': theta_E, 'gamma': gamma, 'e1': e1, 'e2': e2, 'center_x': 0, 'center_y': 0}]
+
+                        theta_E_prime = np.sqrt(2 * q / (1 + q ** 2)) * theta_E
+                        xi = profileAnalysis.mst_invariant_differential(kwargs_lens, radius=theta_E_prime, center_x=None,
+                                                                        center_y=None,
+                                                                        model_list_bool=None, num_points=10)
+                        print(theta_E, gamma, q, 'test')
+                        xi_true = (gamma - 2) / theta_E_prime
+                        npt.assert_almost_equal(xi, xi_true, decimal=2)
+
 
 class TestRaise(unittest.TestCase):
 
